@@ -1,7 +1,7 @@
 /* uuids-table.h : internal interface to `uuids' table
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004, 2009 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -18,8 +18,8 @@
 #ifndef SVN_LIBSVN_FS_UUIDS_TABLE_H
 #define SVN_LIBSVN_FS_UUIDS_TABLE_H
 
-#define APU_WANT_DB
-#include <apu_want.h>
+#define SVN_WANT_BDB
+#include "svn_private_config.h"
 
 #include "svn_io.h"
 #include "svn_fs.h"
